{***************************************************
* (C) 2008 myDanceBot.com, Inc.                    *
* Analyze a four bit counter      *
* AppletImage=scope.gif                            *
****************************************************
Shows how you can analyze activity on the
Propeller's pins using the Logic State Analyzer.

To use this Program with ViewPort, compile and load it into your Propeller's RAM
with the F10 key.  Then start ViewPort and press the "Connect" button.

When ViewPort connects to this program, it will display a logic
analyzer view showing 6 traces.  The program
continually increments a counter and outputs the counter value to the
pins of the Propeller.  The first 4 traces show the pins being toggled
by the counter- they're counting in binary.
You can vary the speed of the counter by turning the dial in the edit section.
You can change the timescale by turning the timescale dial and you can trigger
on a different trace by clicking on its label.  The last 2 traces represent the
data being transmitted and received over the serial connection.}
CON
 _clkmode     = xtal1 + pll16x
 _xinfreq        = 5_000_000         'use standard 5MHz clock
OBJ                                  'include 2 ViewPort objects:
 vp    : "Conduit"                   'transfers data to/from PC
 qs    : "QuickSample"               'samples INA continuously in 1 cog- up to 20Msps
VAR
  long cntr           '              'binary counter
  long nextCycle                     'time for next increment
  long freq                          'counter frequency- edited with ViewPort dial
  long frame[400]                    'stores measurements of INA port
pub demo |a
 vp.register(qs.sampleINA(@frame,1)) 'sample INA into <frame> array
 optional_configure_viewport         'optionally configure viewport's interface   
 vp.share(@freq,@freq)               'share the <freq> variable 
 dira[16..20]~~                      'output pulses on pins 16..20
 nextCycle:=cnt+clkfreq              'initialize first cycle time
 repeat
  outa[20..16]:=(cntr+=1)            'count in binary on pins 20..16
  nextCycle+=((clkfreq/freq)#> 6000 <#clkfreq/10)
  waitcnt(nextCycle)                 'wait until next cycle- controlled by <freq> variable                   
pub optional_configure_viewport  
 vp.config(string("var:io(bits=[cntr[16..19],30tx,31rx]),freq(unit=kHz,f=x/1000,min=1,max=10)"))
     'share just the freq variable.  it has a unit of Hz with range from 1 to 10
     'the io frame is filled by the sampleINA call- we're only interested in specific bits which
     'we name: 16..19 make up the cntr, 30 is tx and 31 is rx  
 vp.config(string("lsa:view=io,timescale=1ms,trigger=io[16]r"))
     'configure the lsa graph to display the io variable with timescale set to 1ms and
     'trigger set on bit 16 rising
 vp.config(string("edit:freq(default=5,mode=[dial,text,scroll])"))
' vp.config(string("edit:freq(default=2000,mode=[scroll,text])"))
    'configure the edit section to display a dial and text for the freq variable- defaults to 5000
 vp.config(string("start:lsa"))
     'start in lsa mode                                                                                                    