{************************************
* ViewPort QuickSample         v4.1 *
* Adjustable 80 Mhz Sampling of INA *
* (C) 2008 myDanceBot.com, Inc.     *
*************************************}{{
Use this Object to take measurements of INA at very high speeds.
Users can use ViewPort to analyze those measurements alongside other variables
from their program.

Features:
-Measure 1440 samples at 32bit up to 80Mhz using 4 cogs.
-Measure 360 samples at 32bit up to 20Mhz using 1 cog.
-Flexible timescale from seconds to nanoseconds
-Edge and Pattern Triggers- can be reset by ViewPort if not Trigger found
-Samples are continuously taken by 1 or 4 interleaved cogs running self modifying assembly code.

Use:
-Include this object in your program:
        Obj qs:"QuickSample"
-Allocate memory for the INAFrame:
        LONG INAFrame[400]
        *Allocate 400 longs for the INAFrame if using 1 cog, or 1600 if using 4.
-Register with ViewPort        
        vp.register(qs.sampleINA(@INAFrame,NumberOfCogs))
        *Use 1 cog to sample up to 20Mhz, or 4 to sample up to 80Mhz }}
CON
 _clkmode        = xtal1 + pll16x
 _xinfreq        = 5_000_000
  SamplesPCog      = 360
pub sampleINA(FrameAddr,nCogs):config 
 {{frameAddr should point to long array of framesamples+16 elements:
 tmask,tstate,tstate1,freq (0=20mhz, 16=5mhz, 32=2.5mhz...),syncA,framsamples of data,syncB
 syncA will be incremented when next measurement is started- syncB will be set to syncA when done
 trigger active when mask<>0. does: waitpeq tstate, waitpeq tstate1- sample
 cog may be restarted remotely from viewport}} 
  long[FrameAddr+16]:=cnt           
  long[FrameAddr+12]:=nCogs-1       
  long[FrameAddr]:=(@MeasureNoI << 2) | (frameAddr<<16)| cognew(@MeasureNoI,FrameAddr)
  if nCogs==4
    cognew(@MeasureNoI,FrameAddr)
    cognew(@MeasureNoI,FrameAddr)
    cognew(@MeasureNoI,FrameAddr)
  return FrameAddr | 2    
dat
              org 0
MeasureNoI    cogid     cogn

              mov       fptr,par 
              rdlong    tmask,fptr
              add       fptr,#4
              rdlong    tstate,fptr              
              add       fptr,#4
              rdlong    tstate1,fptr              
              add       fptr,#4
              rdlong    cps,fptr
              mov       ncog,cps 
              and       ncog,#3
              shr       cps,#2
              add       fptr,#4
              cmp       nsync,#0 wz
    if_z      rdlong    nsync,fptr
    if_z      add       fptr,#4
    if_z      rdlong    xtime,fptr













              add       nsync,cInt
              add       nsync,delay
              add       nsync,sTime
              mov       syncA,fptr
              mov       syncB,fptr

              mov       finc,#4         
              mov       writeloop,wrC2F
              mov       i,#SamplesPCog
              cmp       ncog,#2 wc 
              mov       delay,xTime






    if_c      add       syncB,sAdd1
    if_c      mov       cWait,cps       
    if_c      add       fptr,#4         
    if_c      jmp       #mkTrigger











              shl       delay,#2 
              add       syncB,sAdd4
              cmp       cps,#2    wc,wz  
    if_z      jmp       #do40




              mov       finc,#16
              mov       cWait,#4
              mov       s1,#5
    calcWait  add       cWait,cps'
     if_c     add       cWait,#1
              add       fptr,#4 
              sub       s1,#1          
              djnz      cogn,#calcWait
              
    cWait2    djnz      s1,#cWait2
              tjz       cps,#mFull
              shl       cps,#2
  mkTrigger   tjz       cps,#mFull1
              sub       cps,#17                 wc,nr

              mov       stime,cps
              shl       stime,#9
              mov       measure,storeF
              waitcnt   nsync,0 
              tjz       tmask,#sslow
              waitpeq   tstate, tmask
              waitpeq   tstate1, tmask
sslow         mov       nSync,cnt     
              add       cWait,cnt



store  if_nc  waitcnt   cWait, cps
  measure     nop
              add       measure,d_inc
              djnz      i, #store

writeFrame    mov       i, #SamplesPCog
              wrlong    nsync,syncA
writeloop     nop
              add       fptr, finc
              add       writeloop,d_inc
              djnz      i, #writeloop

              wrlong    nsync,syncB
 lk40         jmp       #MeasureNoI
              long      4      
              long      8
              long      SamplesPCog*8+12
              long      SamplesPCog*8+16
sAdd1         long      SamplesPCog*4+4
sAdd4         long      SamplesPCog*16+4
delay         long      0
stime         long      0
cInt          long      500_000
do40         



              mov       finc,#8
              mov       doL,cogn  
              add       doL,doLook
              mov       cWait,#1
     doL      nop
              add       cWait,s1
              add       fptr,s1
              jmp       #mFull
 doLook       mov       s1,lk40 
syncA         long      0 
syncB         long      0 
nsync         Long      0
fPtr          Long      0
i             LONG      0
d_inc         LONG      $0000_0200
ds_inc        LONG      $0000_0201
cps           LONG      0
cWait         Long      0
finc          Long      0
xtime         long      0

jmpWF         jmp       #writeFrame
storeF        mov       fbufstart, ina               
wrc2f         wrlong    fbufstart, fptr
wrc2f1        wrlong    f1, fptr
dupRead       mov       s1,fbufstart          
modDup        add       s1,d_inc              
movJmp        mov       s1,jmpWF
f1w           mov       f1,ina
f2w           mov       f2,ina
tmask         nop 
tstate1       nop
tstate        nop
mFull1        mov       writeloop,wrC2F1
              mov       f1,f1w
              mov       f2,f2w
mFull         mov       :mk0,dupRead
              mov       :mk1,modDup
              mov       :mkjmp,movJmp
              mov       fbufstart,storeF
:mk0          nop
:mk1          nop
              add       :mk0,ds_inc           
              add       :mk1,d_inc            
              add       :mkjmp,d_inc          
              djnz      i,#:mk0
:mkjmp        nop
              waitcnt   nsync,0    
              tjz       tmask,#f0
              waitpeq   tstate, tmask
              waitpeq   tstate1, tmask
f0            mov       nsync,cnt     
f1            add       cWait,cnt
f2            waitcnt   cWait,#20
              
              fit       $1F0 - SamplesPCog         
fbufstart     nop 
s1            nop
cogn          nop
ncog          nop